## Script (Python) "photo_display_sizes"
##bind container=container
##bind context=
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=max_size=None
##title=returns a dictionary of display sizes
##

# aquire display array from skinpath so projects can easily override 
# you can define the quality of the thumb as third argument
# e.g. 'thumbnail': (128,128,60) would create thumbs with quality 60
return {'thumbnail': (128,128),
        'xsmall': (200,200),
        'small': (320,320),
        'medium': (480,480),
        'large': (768,768),
        'xlarge': (1024,1024)
        }
