
modname     = 'CMFPhoto'
version     = '0.5.0'
# (major, minor, patchlevel, release info) where release info is:
# -99 for alpha, -49 for beta, -19 for rc and 0 for final
# increment the release info number by one e.g. -98 for alpha2
numversion  = (0, 5, 0, 0)

license     = 'GPL'
copyright   = '''(c) 2002-2004 Magnus Heino, Christian Heimes'''

author      = 'Christian Heimes'
author_email= 'heimes@faho.rwth-aachen.de'

short_desc  = 'Zope Content Management Framework Photo product'
long_desc   = '''The CMFPhoto product provides Photo support in your Content
Management System.

This CMF module is compatible with the Plone Content Management System
and includes and external method that needs to be run per plone Site.
'''

copyright_text = '''  CMFPhoto
  Copyright 2002-2003 Magnus Heino, magnus.heino@home.se
  Copyright 2002-2003 Christian Heimes, heimes@faho.rwth-aachen.de

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'''

web         = 'http://www.sourceforge.net/projects/collective'
ftp         = ''
mailing_list= 'collective@lists.sourceforge.net'
bugtracker  = ''

debian_name = 'zope-cmfphoto'
debian_maintainer = 'Christian Heimes'
debian_maintainer_email = ''
debian_add_descr  = '''See the README.Debian to enable CMFPhoto support under Plone.'''
debian_handler    = 'zope'
debian_builddep   = ''
debian_depends    = 'zope (>= 2.6), zope-cmf (>= 1.3), python-imaging|imagemagick'
debian_recommends = 'python-imaging'
debian_suggests   = 'zope-cmfphotoalbum'
debian_conflicts  = ''
debian_replaces   = ''
debian_provides   = ''
